import sys


def sz(x):
    s = hex(x if isinstance(x, int) else len(x))[2:].rjust(4, '0')
    s = bytes.fromhex(s) if sys.version_info[0] == 3 else s.decode('hex')
    return s[::-1]


def pack_uwsgi_vars(var):
    pk = b''
    for k, v in var.items() if hasattr(var, 'items') else var:
        pk += sz(k) + k.encode('utf8') + sz(v) + v.encode('utf8')
    return b'\x00' + sz(pk) + b'\x00' + pk


def parse_addr(addr, default_port=None):
    port = default_port
    if isinstance(addr, str):
        if addr.isdigit():
            addr, port = '', addr
        elif ':' in addr:
            addr, _, port = addr.partition(':')
    elif isinstance(addr, (list, tuple, set)):
        addr, port = addr
    port = int(port) if port else port
    return (addr or '127.0.0.1', port)


def get_host_from_url(url):
    if '//' in url:
        url = url.split('//', 1)[1]
    host, _, url = url.partition('/')
    return (host, '/' + url)



import socket
from .utils import pack_uwsgi_vars, parse_addr, get_host_from_url


def ask_uwsgi(addr_and_port, var, body=''):
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.connect(parse_addr(addr_and_port))
    s.send(pack_uwsgi_vars(var) + body.encode('utf8'))
    response = []
    while 1:
        data = s.recv(4096)
        if not data:
            break
        response.append(data)
    s.close()
    return b''.join(response).decode('utf8')


def curl(addr_and_port, url):
    host, uri = get_host_from_url(url)
    path, _, qs = uri.partition('?')
    host = host or parse_addr(addr_and_port)[0]
    var = {
        'SERVER_PROTOCOL': 'HTTP/1.1',
        'REQUEST_METHOD': 'GET',
        'PATH_INFO': path,
        'REQUEST_URI': uri,
        'QUERY_STRING': qs,
        'SERVER_NAME': host,
        'HTTP_HOST': host,
    }
    return ask_uwsgi(addr_and_port, var)


def cli(*args):
    import argparse
    import sys

    parser = argparse.ArgumentParser()

    parser.add_argument('uwsgi_addr', nargs=1,
                        help='Remote address of uWSGI server')

    parser.add_argument('url', nargs='?', default='/',
                        help='Request URI optionally containing hostname')

    args = parser.parse_args(args or sys.argv[1:])
    print(curl(args.uwsgi_addr[0], args.url))


if __name__ == '__main__':
    cli()



from setuptools import setup

v = '1.0.3'

setup(
    name='uwsgi-tools',
    version=v,
    author='Andrei Fokau',
    author_email='andrei@5monkeys.se',
    url='https://github.com/andreif/uwsgi-tools',
    download_url='https://github.com/andreif/uwsgi-tools/tarball/%s' % v,
    license='MIT',
    description='uwsgi tools: curl and reverse proxy',
    long_description='''
        Reverse proxy and curl for uWSGI server using `uwsgi` protocol
    ''',
    packages=['uwsgi_tools'],
    entry_points={
        'console_scripts': [
            'uwsgi_proxy = uwsgi_tools.proxy:cli',
            'uwsgi_curl = uwsgi_tools.curl:cli',
        ]
    },
    install_requires=[],
)
